@@ -6,18 +6,18 @@ Protocol objects representing different implementations of the same classes.
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Callable, Dict  # drop with Python 3.8
-from typing import Generator, Mapping, Protocol, Sequence, Union
+from typing import TYPE_CHECKING, Any, Protocol, TypeAlias, Union
+from collections.abc import Callable, Generator, Mapping, Sequence
 
 from . import pq
 from ._enums import PyFormat as PyFormat
-from ._compat import LiteralString, TypeAlias, TypeVar
+from ._compat import LiteralString, Template, TypeVar
 
 if TYPE_CHECKING:
-    from . import sql  # noqa: F401
+    from . import sql
     from .rows import Row, RowMaker
     from .pq.abc import PGresult
-    from .waiting import Ready, Wait  # noqa: F401
+    from .waiting import Ready, Wait
     from ._adapters_map import AdaptersMap
     from ._connection_base import BaseConnection
 
@@ -26,13 +26,14 @@ NoneType: type = type(None)
 # An object implementing the buffer protocol
 Buffer: TypeAlias = Union[bytes, bytearray, memoryview]
 
-Query: TypeAlias = Union[LiteralString, bytes, "sql.SQL", "sql.Composed"]
+QueryNoTemplate: TypeAlias = Union[LiteralString, bytes, "sql.SQL", "sql.Composed"]
+Query: TypeAlias = Union[QueryNoTemplate, Template]
 Params: TypeAlias = Union[Sequence[Any], Mapping[str, Any]]
 ConnectionType = TypeVar("ConnectionType", bound="BaseConnection[Any]")
 PipelineCommand: TypeAlias = Callable[[], None]
-DumperKey: TypeAlias = Union[type, "tuple[DumperKey, ...]"]
+DumperKey: TypeAlias = Union[type, tuple["DumperKey", ...]]
 ConnParam: TypeAlias = Union[str, int, None]
-ConnDict: TypeAlias = Dict[str, ConnParam]
+ConnDict: TypeAlias = dict[str, ConnParam]
 ConnMapping: TypeAlias = Mapping[str, ConnParam]
 
 
@@ -40,7 +41,7 @@ ConnMapping: TypeAlias = Mapping[str, ConnParam]
 
 RV = TypeVar("RV")
 
-PQGenConn: TypeAlias = Generator["tuple[int, Wait]", "Ready | int", RV]
+PQGenConn: TypeAlias = Generator[tuple[int, "Wait"], "Ready | int", RV]
 """Generator for processes where the connection file number can change.
 
 This can happen in connection and reset, but not in normal querying.
@@ -56,14 +57,12 @@ class WaitFunc(Protocol):
     Wait on the connection which generated `PQgen` and return its final result.
     """
 
-    def __call__(
-        self, gen: PQGen[RV], fileno: int, interval: float | None = None
-    ) -> RV: ...
+    def __call__(self, gen: PQGen[RV], fileno: int, interval: float = ...) -> RV: ...
 
 
 # Adaptation types
 
-DumpFunc: TypeAlias = Callable[[Any], "Buffer | None"]
+DumpFunc: TypeAlias = Callable[[Any], Buffer | None]
 LoadFunc: TypeAlias = Callable[[Buffer], Any]
 
 
@@ -71,7 +70,7 @@ class AdaptContext(Protocol):
     """
     A context describing how types are adapted.
 
-    Example of `~AdaptContext` are `~psycopg.Connection`, `~psycopg.Cursor`,
+    Example of `!AdaptContext` are `~psycopg.Connection`, `~psycopg.Cursor`,
     `~psycopg.adapt.Transformer`, `~psycopg.adapt.AdaptersMap`.
 
     Note that this is a `~typing.Protocol`, so objects implementing
